searchNodes=[{"ref":"Read.html","title":"Read","type":"module","doc":"Provides utilities to implement and work with Readable types"},{"ref":"Read.html#defreadable/3","title":"Read.defreadable/3","type":"macro","doc":"Helper to define readable relation for pair of types. Accepts: target type which we want to read source arg expression :: type pair block of code where relation is described Examples iex&gt; quote do ...&gt; import Read ...&gt; defreadable URI, from: x :: BitString do ...&gt; URI.parse(x) ...&gt; end ...&gt; end ...&gt; |&gt; Code.compile_quoted iex&gt; import Read iex&gt; read(&quot;https://hello.world&quot;, URI) %URI{ authority: &quot;hello.world&quot;, fragment: nil, host: &quot;hello.world&quot;, path: nil, port: 443, query: nil, scheme: &quot;https&quot;, userinfo: nil } iex&gt; quote do ...&gt; import Read ...&gt; defreadable NaiveDateTime, from: x :: Tuple do ...&gt; case NaiveDateTime.from_erl(x) do ...&gt; {:ok, y} -&gt; y ...&gt; {:error, _} -&gt; fail!(x) ...&gt; end ...&gt; end ...&gt; end ...&gt; |&gt; Code.compile_quoted iex&gt; import Read iex&gt; read({{2000, 1, 1}, {13, 30, 15}}, NaiveDateTime) ~N[2000-01-01 13:30:15] iex&gt; read({{2000, 13, 1}, {13, 30, 15}}, NaiveDateTime) ** (Readable.Exception) NaiveDateTime can not be read from {{2000, 13, 1}, {13, 30, 15}}"},{"ref":"Read.html#read/2","title":"Read.read/2","type":"function","doc":"Helper to read expression into given type Examples iex&gt; Read.read(&quot;1&quot;, Integer) 1"},{"ref":"Readable.html","title":"Readable","type":"protocol","doc":"Protocol which describes type convertion a -&gt; b. Usually @type a :: String.t() but there is no any artificial restriction regarding type a."},{"ref":"Readable.html#read/1","title":"Readable.read/1","type":"function","doc":"Accepts struct with field :data and type pair information in structure name and returns value of target type or raise exception."},{"ref":"Readable.html#t:t/0","title":"Readable.t/0","type":"type","doc":""},{"ref":"Readable.Exception.html","title":"Readable.Exception","type":"exception","doc":""},{"ref":"Readable.From.BitString.To.Atom.html","title":"Readable.From.BitString.To.Atom","type":"module","doc":""},{"ref":"Readable.From.BitString.To.BitString.html","title":"Readable.From.BitString.To.BitString","type":"module","doc":""},{"ref":"Readable.From.BitString.To.Float.html","title":"Readable.From.BitString.To.Float","type":"module","doc":""},{"ref":"Readable.From.BitString.To.Integer.html","title":"Readable.From.BitString.To.Integer","type":"module","doc":""},{"ref":"Readable.From.BitString.To.PID.html","title":"Readable.From.BitString.To.PID","type":"module","doc":""},{"ref":"Readable.From.BitString.To.Port.html","title":"Readable.From.BitString.To.Port","type":"module","doc":""},{"ref":"Readable.From.BitString.To.Reference.html","title":"Readable.From.BitString.To.Reference","type":"module","doc":""},{"ref":"readme.html","title":"Readable","type":"extras","doc":"Readable Elixir language have standard String.Chars protocol which describes T -&gt; String relation (like Haskell Show type class). But Elixir don&#39;t have any standard protocol for String -&gt; T relation (some equivalent of Haskell Read type class). But I think such protocol completely make sense. This protocol describes read relation between 2 types. Instance of type B can be read from instance of type A. Usually type A = String, but this implementation don&#39;t have this constraint."},{"ref":"readme.html#example","title":"Readable - Example","type":"extras","doc":"import Read defreadable Integer, from: x :: BitString do String.to_integer(x) end defreadable URI, from: x :: BitString do URI.parse(x) end defreadable NaiveDateTime, from: x :: Tuple do case NaiveDateTime.from_erl(x) do {:ok, y} -&gt; y {:error, _} -&gt; fail!(x) end end and then we can use implementation of Read type class iex&gt; Read.read(&quot;123&quot;, Integer) 123 iex&gt; Read.read(&quot;https://hello.world&quot;, URI) %URI{ authority: &quot;hello.world&quot;, fragment: nil, host: &quot;hello.world&quot;, path: nil, port: 443, query: nil, scheme: &quot;https&quot;, userinfo: nil } iex&gt; Read.read({{2000, 1, 1}, {13, 30, 15}}, NaiveDateTime) ~N[2000-01-01 13:30:15] iex&gt; Read.read({{2000, 13, 1}, {13, 30, 15}}, NaiveDateTime) ** (Readable.Exception) NaiveDateTime can not be read from {{2000, 13, 1}, {13, 30, 15}} iex&gt; Read.read(&quot;https://hello.world&quot;, TypeNotExist) ** (ArgumentError) argument error :erlang.binary_to_existing_atom(&quot;Elixir.Readable.From.BitString.To.TypeNotExist&quot;, :utf8) (elixir) src/elixir_aliases.erl:119: :elixir_aliases.safe_concat/1 (readable) lib/read.ex:84: Read.read/2"},{"ref":"readme.html#installation","title":"Readable - Installation","type":"extras","doc":"The package can be installed by adding readable to your list of dependencies in mix.exs: def deps do [ {:readable, &quot;~&gt; 0.1.0&quot;} ] end"}]